var GpsMap = {
    height: 180,
    width: 180,
    canvas: null,
    ctx: null,
    imageData: [],
    buffArray: [],
    buff8: [],
    buff32: []
};
GpsMap.init = function(){
    this.canvas = document.getElementById("cvMap");
    this.ctx = this.canvas.getContext("2d");
    this.canvas.width = this.width;
    this.canvas.height = this.height;
    this.imageData = this.ctx.createImageData(this.width, this.height);
    this.buffArray = new ArrayBuffer(this.width * this.height * 4);
    this.buff8 = new Uint8Array(this.buffArray);
    this.buff32 = new Uint32Array(this.buffArray);
};
GpsMap.draw = function(){
    for(let row = 0; row < this.height; row++){
        let y = ((Camera.y - this.height / 2)  + row) & (ImgMap.height - 1);
        for(let col = 0; col < this.width; col++){
            let x = ((Camera.x - this.width / 2) + col) & (ImgMap.width - 1);
            let offset = (y << 10) + x;
            this.buff32[row * this.width + col] = ImgMap.colour[offset];
        }
    }
    this.drawGraticule();
    this.imageData.data.set(this.buff8);
    this.ctx.putImageData(this.imageData, 0, 0);
};
GpsMap.drawGraticule = function(){
    const strt = Math.floor(this.width / 2) - 16;
    const end = strt + 30;
    const black = 0xff000000 | 0;
    for(let row = strt; row <= end; row++){
        if(row > strt + 10 && row < strt + 20){continue;}
        if(row === strt || row === end){
            for(let col = strt; col <= end; col++){
                if(col > strt + 10 && col < strt + 20){continue;}
                this.buff32[row * this.width + col] = black; //0xff000000
            }
        } else {
            let x = strt;
            this.buff32[row * this.width + x] = black; //0xff000000;
            x = end;
            this.buff32[row * this.width + x] = black; //0xff000000;
        }
    }
};
var Heading = {
    canvas: [], // array to hold both
    ctx: [],
    image: new Image(),
    width: 180,
    height: 180
};
Heading.init = function(){
    this.canvas.push(document.getElementById("cvHeadA"));
    this.canvas.push(document.getElementById("cvHeadB"));
    for(let h = 0; h < 2; h++){
        this.ctx[h] = this.canvas[h].getContext("2d");
        this.canvas[h].width = this.width;
        this.canvas[h].height = this.height;
    }
    let img = new Image();
    img.onload = function() {
        Heading.ctx[1].drawImage(this, 0, 0);
    };
    img.src = "images/headingB.png";
    this.image.src = "images/headingA.png";
};
Heading.draw = function(){
    let angle = (Camera.angle + Math.PI * 0.5) % (Math.PI * 2);
    this.ctx[0].clearRect(0,0,this.width, this.height);
    this.ctx[0].save();
    let mid = this.width / 2;
    this.ctx[0].translate(mid, mid);
    this.ctx[0].rotate(angle);
    this.ctx[0].drawImage(this.image, -mid, -mid);
    this.ctx[0].restore();
};
var Pitch = {
    image: null,
    value: -90
};
Pitch.init = function(){
    this.image = document.getElementById("imgPitchB");
};
Pitch.draw = function(){
    // move the sky/earth image up and down
    let set = (Camera.horizon - 100)/5; // arbitrary
    this.image.style.top = "" + (this.value + set) + "px";
};
var Altitude = {
    canvas: null,
    ctx: null,
    width: 180,
    height: 180
};
Altitude.init = function(){
    this.canvas = document.getElementById("cvAlt");
    this.ctx = this.canvas.getContext("2d");
    this.canvas.width = this.width;
    this.canvas.height = this.height;
};
Altitude.draw = function(){
    let hasl = Math.round(Camera.height * 50 / 3);
    if(hasl > 9999){hasl = 9999;}
    let ts = hasl/1000; // number of thousands
    let tsa = Math.PI * 2 * (ts / 10) - Math.PI * 0.5;
    let us = hasl % 1000; // remaining units
    let usa = Math.PI * 2 * (us / 1000) - Math.PI * 0.5;
    this.ctx.clearRect(0,0,this.width,this.height);
    this.drawHand(tsa, 58, 3);
    this.drawHand(tsa, 60, 1);
    this.drawHand(usa, 78, 3);
    this.drawHand(usa, 80, 1);
};
Altitude.drawHand = function(angle, length, width){
    const midPt = 90;
    let ls = findPointOnCircle(midPt, midPt, 4, angle);
    let lf = findPointOnCircle(midPt,midPt,length,angle);
    this.ctx.beginPath();
    this.ctx.moveTo(ls.x, ls.y);
    this.ctx.lineTo(lf.x, lf.y);
    this.ctx.strokeStyle = "red";
    this.ctx.lineWidth = width;
    this.ctx.stroke();
    this.ctx.closePath();
};
var FlightData = {
    lblTime: null,
    time: Date.now(),
    frames: 0,
    lblFps: null
};
FlightData.init = function(){
    this.lblTime = document.getElementById("lblTime");
    this.lblFps = document.getElementById("lblFps");
};
FlightData.draw = function(){
    this.frames++;
    let time = Date.now();
    if(time - this.time > 1000){
        let fps = this.frames / ((time - this.time) / 1000);
        this.lblFps.textContent = fps.toFixed(1);
        this.frames = 0;
        this.time = time;
        let clock = new Date();
        this.lblTime.textContent = clock.toTimeString().substr(0, 8); // works with date not set
    }
};