var Viewer = {
    projectionWidth: 800,
    projectionHeight: 500,
    fov: 90 * Math.PI / 180,
    canvas: null,
    ctx: null,
    imageData: [],
    buffArray: [],
    buff8: [],
    buff32: [],
    cameraDistance: 0,
    mapWidth: 0,
    mapHeight: 0,
    hidden: null
 };
 Viewer.init = function(canv){
    this.canvas = document.getElementById(canv);
    this.ctx = this.canvas.getContext("2d");
    this.canvas.width = this.projectionWidth;
    this.canvas.height = this.projectionHeight;
    this.imageData = this.ctx.createImageData(this.projectionWidth, this.projectionHeight);
    this.buffArray = new ArrayBuffer(this.projectionWidth * this.projectionHeight * 4);
    this.buff8 = new Uint8Array(this.buffArray);
    this.buff32 = new Uint32Array(this.buffArray);
    this.cameraDistance = Math.floor((this.projectionWidth / 2) / Math.tan(this.fov / 2)) * 2;
    this.mapWidth = ImgMap.width - 1;
    this.mapHeight = ImgMap.height - 1;
    this.hidden = new Int16Array(this.projectionWidth);
 };
 Viewer.drawBackground = function(){
    let c = 0xffddc275; //ABGR as int byte order reverse of RGBA [] colour order
    for(let i = 0; i < this.buff32.length; i++){
        this.buff32[i] = c;
    }
 };
Viewer.renderView = function(){
    this.drawBackground();
    let sina = Math.sin(Camera.angle);
    let cosa = Math.cos(Camera.angle);
    for (let i = 0; i < this.hidden.length; i++){
        this.hidden[i] = this.projectionHeight;
    }
    let deltaZ = 1;
    for(let z = 1; z < this.cameraDistance; z += deltaZ){
        let leftX = -cosa * z - sina * z;
        let rightX = cosa * z - sina * z;
        let leftY = sina * z - cosa * z;
        let rightY = -sina * z - cosa * z;
        let dx = (rightX - leftX) / this.projectionWidth;
        let dy = (rightY - leftY) / this.projectionWidth;
        leftX += Camera.x;
        leftY += Camera.y;
        let invZ = 1 / z * 240; // decreases as z increases
        for(let i = 0; i < this.projectionWidth; i++){
            let offset = Math.floor((Math.floor(leftY) & this.mapWidth) << 10) + (Math.floor(leftX) & this.mapHeight);
            let drawHeight = Math.floor((Camera.height - ImgMap.altitude[offset]) * invZ + Camera.horizon);
            if(drawHeight < 0){drawHeight = 0;}// terraine above viewer top
            if(drawHeight <= this.hidden[i]){
                this.drawVerticalStrip(i, drawHeight, this.hidden[i], ImgMap.colour[offset]);
                this.hidden[i] = drawHeight;
            } // else hidden so not drawn
            leftX += dx;
            leftY += dy;
        }
        deltaZ += 0.005;
    }
    this.updateView();
};
Viewer.drawVerticalStrip = function(x, yTop, yBottom, colour){
    let offset = ((yTop * Viewer.projectionWidth) + x); // position of first pixel to draw
    for ( let i = yTop; i < yBottom; i++){
       Viewer.buff32[offset] = colour;
       offset += Viewer.projectionWidth; // position of next vertical pixel
    }
};
Viewer.updateView = function(){
    this.imageData.data.set(this.buff8);
    this.ctx.putImageData(this.imageData, 0, 0);
};

var Camera = {
    x:        512, // x position on the map
    y:        800, // y position on the map
    height:    78, // height of the camera
    angle:      0, // direction of the camera
    horizon:  100 // horizon position (look up and down)
};

Camera.setAngle = function() {
    this.angle += flight.rotate * 0.1;
    this.x -= flight.forward * Math.sin(this.angle);
    this.y -= flight.forward * Math.cos(this.angle);
    this.height += flight.climb;
    this.horizon += flight.pitch;
    let offset = ((Math.floor(this.y) & (ImgMap.width-1)) << 10) + (Math.floor(this.x) & (ImgMap.height-1))|0;
    if ((ImgMap.altitude[offset]+10) > this.height) {
        this.height = ImgMap.altitude[offset] + 10; // dont crash
    }
};
var ImgMap = {
    width: 1024,
    height: 1024,
    colour: new Uint32Array(1024*1024), // array of 32 bit colour values
    altitude: new Uint8Array(1024*1024) // byte array of height values 0 to 255
};
ImgMap.loadImages = function(imgList){
    return new Promise(function(resolve, reject){
        let ret = [];
        let count = imgList.length;
        for(let i = 0; i < imgList.length; i++){
            let img = new Image();
            img.onload = function() {
                let canvas = document.createElement("canvas");
                let ctx = canvas.getContext("2d");
                canvas.width = this.width;
                canvas.height = this.height;
                ctx.drawImage(img, 0, 0, this.width, this.height);
                ret[i] = ctx.getImageData(0, 0, this.width, this.height).data;
                count--;
                if(count === 0){resolve(ret);}
            };
            img.src = imgList[i];
        }
    });
};
ImgMap.load = async function(imgList){
    let result = await this.loadImages(imgList);
    let colourData = result[0], heightData = result[1];
    let sqr = this.width * this.height;
    for(let i = 0; i < sqr; i++){
        // we want all 4 components in the colour array RGBA
        this.colour[i] = 0xff000000 | (colourData[(i<<2) + 2] << 16) | (colourData[(i<<2) + 1] << 8) | colourData[(i<<2)];
        this.altitude[i] = heightData[i<<2];
    }
    reDraw(); // start things running
};